/****************************************************************************/
/*                       24HOP 2015 - Summit Preview                        */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                  Heap Tables: Forwarding Pointers                        */
/****************************************************************************/
use SqlServerInternals
go

set nocount on
go

-- Forwarding pointers
if exists(
	select *
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		t.name = 'HeapTable' and s.name = 'dbo'
)
	drop table dbo.HeapTable
go

create table dbo.HeapTable
(
	Placeholder char(100) not null
		constraint DEF_HeapTable_Placeholder
		default 'Placeholder',
	IntVal int not null,
	Data varchar(255) null
)
go

;with n1(c) as (select 0 union all select 0) -- 2 rows
,n2(c) as (select 0 from n1 as t1 cross join n1 as t2) -- 4 rows
,n3(c) as (select 0 from n2 as t1 cross join n2 as t2) -- 16 rows
,n4(c) as (select 0 from n3 as t1 cross join n3 as t2) -- 256 rows
,n5(c) as (select 0 from n4 as t1 cross join n4 as t2) -- 65,536 rows
,n6(c) as (select 0 from n5 as t1 cross join n1 as t2) -- 131,072 rows
,ids(id) as (select row_number() over (order by (select null)) from n6)
insert into dbo.HeapTable(IntVal)
	select id
	from Ids
go

select page_count, forwarded_record_count, * 
from sys.dm_db_index_physical_stats(DB_ID(), OBJECT_ID(N'dbo.HeapTable'),0,null,'DETAILED')
go

set statistics io, time on
select count(*) from dbo.HeapTable
set statistics io, time off
go

update dbo.HeapTable
set Data = REPLICATE('a',255)
go

-- 6330	94150 - 100480
select page_count, forwarded_record_count, * 
from sys.dm_db_index_physical_stats(DB_ID(), OBJECT_ID(N'dbo.HeapTable'),0,null,'DETAILED')
go

set statistics io, time on
select count(*) from dbo.HeapTable
set statistics io, time off
go

update dbo.HeapTable
set Data = null
go

select page_count, forwarded_record_count, * 
from sys.dm_db_index_physical_stats(DB_ID(), OBJECT_ID(N'dbo.HeapTable'),0,null,'DETAILED')
go